package Controller;

use strict;
use vars qw( $DefaultSAXHandler $DefaultSAXGenerator);
use XML::LibXML;
use XML::SAX::Machines qw( :all );

$DefaultSAXHandler   ||= 'StringWriter';
$DefaultSAXGenerator ||= 'XML::SAX::ParserFactory';

sub new {
    my $class = shift;
    my %args = @_;

    my $self;
    if ( defined $args{Handler} ) {
        if ( ! ref( $args{Handler} ) ) {
            my $handler_class =  $args{Handler};   
            eval "require $handler_class";
            $args{Handler} = $handler_class->new();
        }
    }
    else {  
        eval "require $DefaultSAXHandler";
        $args{Handler} = $DefaultSAXHandler->new();  
    }
    
    if ( defined $args{Generator} ) {
        if ( ! ref( $args{Generator} ) ) {
            my $driver_class =  $args{Generator};
            eval "use $driver_class";
            $args{Generator} = $driver_class->new();
        }
    }
    else {
        eval "use $DefaultSAXGenerator";
        $args{Generator} = $DefaultSAXGenerator->new();
    }
    
    $args{FilterList} ||= [];
        
    $self = bless \%args, $class;
            
    return $self;
}

sub parse {
    my $self = shift;
    my $to_be_parsed = shift;
    
    # filters passed to the object from the application side.
    my @filterlist    = @{$self->{FilterList}} || ();
            

    # give the Generator a peek at what it's about to parse and alter it, if needed.
    if ( $self->{Generator}->can('prepare') ) {
      $to_be_parsed = $self->{Generator}->prepare( $to_be_parsed );
    }
    
    # allow filters to be passed from the generator
    # (could be hard-coded, or filters set during prepare()).

    if ( $self->{Generator}->can('get_filters') ) {
        push @filterlist, $self->{Generator}->get_filters;
    }
    
    # build the filter machine, setting the last stage to the passed Handler
    my $machine = Pipeline( @filterlist, $self->{Handler} );    
    # set the generator to fire its events at the pipeline
    $self->{Generator}->set_handler( $machine );
            
    # get the result and return it.
    my $parse_result = $self->{Generator}->parse( $to_be_parsed );
    return $parse_result;
}

####
# Accessors
####
         
sub set_handler {
    my $self = shift;
    my $handler = shift;
    if ( defined( $handler ) ) {
        if ( ! ref( $handler ) ) {
            my $handler_class =  $handler;
            eval "use $handler_class";
            $self->{Handler} = $handler_class->new();
        }
        else {
            $self->{Handler} = $handler;
        }
    }
}

sub set_generator {
    my $self = shift;
    my $generator = shift;   
    if ( defined( $generator ) ) {
        if ( ! ref( $generator ) ) {
            my $generator_class =  $generator;
            eval "use $generator_class";
            $self->{Generator} = $generator_class->new();
        }
        else {   
            $self->{Generator} = $generator;
        }
    }
}
            
sub set_filterlist {
    my $self = shift;
    my @filters = @_;
    $self->{FilterList} = \@filters;
}

1;